@@ -1,10 +1,13 @@
+import warnings
+warnings.filterwarnings("ignore")
+
 import os
 import pickle
 import csv 
-from .dataCleaning import *
-from .taxonomyAPI import *
-from .higherLevelAPI import *
-from .commonNameAPI import *
+from dataCleaning import *
+from taxonomyAPI import *
+from higherLevelAPI import *
+from commonNameAPI import *
 import itertools
 
 class EcoNameTranslator:
@@ -24,6 +27,7 @@ class EcoNameTranslator:
         self.mapKnownTaxaLevelToSpecies()
         print("Trying common name translation...(this may take a while)")
         self.tryCommonNameOnErrorCases()
+        self.ensureUniqueInIndex()
         return self.index
     
     def tagInputWithTaxaLevel(self):
@@ -35,14 +39,14 @@ class EcoNameTranslator:
         names = list(map(lambda x: self.index[x][0], self.index.keys()))
         taxaTuples = list(map(lambda x: self.index[x][1], self.index.keys()))
         originalNamesWthTaxa = list(zip(names,taxaTuples))
-        speciesNamesOnly = list(map(higherLevelAPI,originalNamesWthTaxa))
+        speciesNamesOnly = higherLevelAPI(originalNamesWthTaxa)
         self.writeApiResultsToIndex(speciesNamesOnly)
     
     def tryCommonNameOnErrorCases(self):
         names = list(map(lambda x: self.index[x][0], self.index.keys()))
         currentSpeciesResults = list(map(lambda x: self.index[x][1], self.index.keys()))
         bestEffortOfSpecies = list(zip(names,currentSpeciesResults))
-        speciesNamesOnly = list(map(commonNameAPI,bestEffortOfSpecies))
+        speciesNamesOnly = commonNameAPI(bestEffortOfSpecies)
         self.writeApiResultsToIndex(speciesNamesOnly)
     
     def writeApiResultsToIndex(self,taxonomyAPIResuls):
@@ -53,6 +57,12 @@ class EcoNameTranslator:
         for name in self.index:
             try: self.index[name][1] = taxaAPIDict[self.index[name][0]]
             except: pass
+        
+    def ensureUniqueInIndex(self):
+        for name in self.index:
+            species = self.index[name][1]
+            species = list(set(species))
+            self.index[name][1] = species
     
     def toPickleFile(self,path):
         with open(path,'wb') as fh:
@@ -61,9 +71,9 @@ class EcoNameTranslator:
     def toCSV(self,path,name="translatedNames.csv"):
         headers = [["Input Name","Name After Cleaning","Result"]]
         csvData = map(lambda indivEcoName: \
-                        list(map(lambda result: [indivEcoName,self.index[indivEcoName][0],result], \
-                        self.index[indivEcoName][1])) \
-                      ,self.index.keys())
+                    list(map(lambda result: [indivEcoName,self.index[indivEcoName][0],result], \
+                    self.index[indivEcoName][1])), \
+                  self.index.keys())
                       
         csvData = list(itertools.chain(headers,*csvData))
         
