@@ -1,23 +1,49 @@
-import requests 
+from gevent import monkey as curious_george
+curious_george.patch_all(select=False)
+import grequests 
+import itertools
 
 def taxonomyAPI(names):
-    politeness = 50
-    responses = []
-    excepted = []
-    for i in range(0,len(names),politeness):
-        print(f"Processing Records {str(i)} to {str(min(i+politeness,len(names)))} [of {len(names)}]")
-        apiInputWithCapitalisedGenus = list(map(lambda x: x.capitalize(),names[i:i+politeness]))
-        succeeded,failed = callTaxaAPI(apiInputWithCapitalisedGenus)
-        responses.extend(succeeded)
-        excepted.extend(failed)
+    capNames = list(map(lambda x: x.capitalize(),names))
+    reqs = packageReqs(capNames)
+    urls = list(map(prepUrl,reqs))
+    readyRequests = (grequests.get(url) for url in urls)
+    results = grequests.map(readyRequests,size=10)
+    succeeded,excepted = groupToSuccessesAndFailures(results,reqs)
+    newSucceeded, newExcepted = individuallyMakeRequests(excepted.split("|"))
+    succeeded.extend(newSucceeded)
+    speciesResponses = list(map(processSingleResponse,succeeded))
+    return speciesResponses,newExcepted
 
-    speciesResponses = list(map(processSingleResponse,responses))
-    return speciesResponses,excepted
+def groupToSuccessesAndFailures(results,reqs):
+    succeeded = []
+    excepted = []
+    for i in range(len(results)):
+        try: 
+            packagedResults = results[i].json()['data']
+            if len(packagedResults) != (reqs[i].count("|")+1): raise ValueError()
+            succeeded.extend(packagedResults)
+        except: excepted.append(reqs[i])
+    excepted = "|".join(excepted)
+    return succeeded,excepted
+
+def packageReqs(names):
+    bulked = []
+    politeness = 50
+    for i in range(0,len(names),politeness):
+        bulked.append("|".join(names[i:i+politeness]))
+    return bulked
+
+def prepUrl(req):
+    return f'http://resolver.globalnames.org/name_resolvers.json?names={req}'
 
 def processSingleResponse(response):
-    if response['is_known_name']: return (response['supplied_name_string'].lower(), True, parseSingleTaxonomyFromAPI(response))
-    elif 'results' in response: return (response['supplied_name_string'].lower(),False, parseSingleTaxonomyFromAPI(response))
-    return (response['supplied_name_string'], False, ('',''))
+    try:
+        if response['is_known_name']: return (response['supplied_name_string'].lower(), True, parseSingleTaxonomyFromAPI(response))
+        elif 'results' in response: return (response['supplied_name_string'].lower(),False, parseSingleTaxonomyFromAPI(response))
+        return (response['supplied_name_string'], False, ('',''))
+    except:
+        return('hello',False,('',''))
 
 def parseSingleTaxonomyFromAPI(taxonomicAPIres):
     dataFromMultipleSources = taxonomicAPIres['results']
@@ -65,7 +91,9 @@ def individuallyMakeRequests(names):
 
 def makeCall(preparedAPILoad):
     try:
-        apiResult = requests.get(f'http://resolver.globalnames.org/name_resolvers.json?names={preparedAPILoad}').json()['data']
-        return (True,apiResult)
+        reqs = (grequests.get(url) for url in [f'http://resolver.globalnames.org/name_resolvers.json?names={preparedAPILoad}'])
+        results = grequests.map(reqs,size=1)
+        res = ([r.json()['data'] for r in results][0])
+        return (True,res)
     except Exception as e: 
         return (False,[])
